/*
	This code is part of the MCJoin project.
	Authored by Steven Begley (sbegley@latrobe.edu.au) as part of my PhD candidature.
	La Trobe University,
	Melbourne, Australia.
*/
#pragma once

#include "CRelation.h"
#include "CFlipFlop.h"
#include "DataTypes.h"

class CFlipFlopBuffer
{
public:
	CFlipFlopBuffer(const CRelation* input, uint32 radixLength, uint32 maxDLength) : _input(input), _radixLength(radixLength), _maxDLength(maxDLength), _source(NULL), _destination(NULL)
	{		
	}

	~CFlipFlopBuffer()
	{
		if(_canDeleteSource)
		{
			delete _source;
		}
	}

	CRelation* doPartitioning(uint32 startIndex, uint32 endIndex, uint32 baseIndex, bool retainPayload)
	{
		uint32 D(0);
		uint32 bitsProcessed(0);
		CFlipFlop ff;
		const CRelation* tmp(NULL);
		uint32 itemCount(endIndex - startIndex);

		_source = _input;		
		_canDeleteSource = false;

		while(bitsProcessed != _radixLength)
		{
			D = _maxDLength;
			if( (D + bitsProcessed) > _radixLength)
			{
				D = _radixLength - bitsProcessed;
			}

			//
			if(_destination == NULL)
			{
				_destination = new CRelation(itemCount);
			}			

			ff.setRelations(_source, _destination, itemCount);

			if(bitsProcessed == 0)
			{
				ff.go(D, bitsProcessed, startIndex, endIndex, baseIndex, retainPayload);
			}
			else
			{
				ff.go(D, bitsProcessed);
			}
			//

			bitsProcessed += D;

			if(bitsProcessed != _radixLength)
			{
				// Switch the pointers
				tmp = _source;
				_source = _destination;
				_destination = const_cast<CRelation *>(tmp);

				if(_canDeleteSource == false)
				{
					_destination = new CRelation(itemCount);
					_canDeleteSource = true;
				}				
			}
		}

		return _destination;
	}

private:
	const CRelation* _input; 

	const CRelation* _source;
	CRelation* _destination;

	uint32 _radixLength;
	uint32 _maxDLength;

	bool _canDeleteSource;
};